@@ -5,11 +5,11 @@
  *  Copyright (C) 2016 SchedMD LLC
  *  Written by Danny Auble da@schedmd.com, et. al.
  *
- *  This file is part of SLURM, a resource management program.
+ *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
  *  Please also read the included file: DISCLAIMER.
  *
- *  SLURM is free software; you can redistribute it and/or modify it under
+ *  Slurm is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
@@ -25,13 +25,13 @@
  *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
  *
- *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
+ *  Slurm is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with SLURM; if not, write to the Free Software Foundation, Inc.,
+ *  with Slurm; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
@@ -50,6 +50,7 @@
 #include "src/common/slurm_auth.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurmdbd_defs.h"
+#include "src/common/slurmdbd_pack.h"
 #include "src/common/xsignal.h"
 #include "slurm_persist_conn.h"
 
@@ -614,6 +615,7 @@ extern int slurm_persist_conn_open(slurm_persist_conn_t *persist_conn)
 		if (resp && (rc == SLURM_SUCCESS)) {
 			rc = resp->rc;
 			persist_conn->version = resp->ret_info;
+			persist_conn->flags |= resp->flags;
 		}
 
 		if (rc != SLURM_SUCCESS) {
@@ -1069,9 +1071,19 @@ extern void slurm_persist_free_init_req_msg(persist_init_req_msg_t *msg)
 extern void slurm_persist_pack_rc_msg(
 	persist_rc_msg_t *msg, Buf buffer, uint16_t protocol_version)
 {
-	packstr(msg->comment, buffer);
-	pack32(msg->rc, buffer);
-	pack16(msg->ret_info, buffer);
+	if (protocol_version >= SLURM_18_08_PROTOCOL_VERSION) {
+		packstr(msg->comment, buffer);
+		pack16(msg->flags, buffer);
+		pack32(msg->rc, buffer);
+		pack16(msg->ret_info, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		packstr(msg->comment, buffer);
+		pack32(msg->rc, buffer);
+		pack16(msg->ret_info, buffer);
+	} else {
+		error("%s: invalid protocol version %u",
+		      __func__, protocol_version);
+	}
 }
 
 extern int slurm_persist_unpack_rc_msg(
@@ -1083,9 +1095,20 @@ extern int slurm_persist_unpack_rc_msg(
 
 	*msg = msg_ptr;
 
-	safe_unpackstr_xmalloc(&msg_ptr->comment, &uint32_tmp, buffer);
-	safe_unpack32(&msg_ptr->rc, buffer);
-	safe_unpack16(&msg_ptr->ret_info, buffer);
+	if (protocol_version >= SLURM_18_08_PROTOCOL_VERSION) {
+		safe_unpackstr_xmalloc(&msg_ptr->comment, &uint32_tmp, buffer);
+		safe_unpack16(&msg_ptr->flags, buffer);
+		safe_unpack32(&msg_ptr->rc, buffer);
+		safe_unpack16(&msg_ptr->ret_info, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		safe_unpackstr_xmalloc(&msg_ptr->comment, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->rc, buffer);
+		safe_unpack16(&msg_ptr->ret_info, buffer);
+	} else {
+		error("%s: invalid protocol_version %u",
+		      __func__, protocol_version);
+		goto unpack_error;
+	}
 
 	return SLURM_SUCCESS;
 
@@ -1122,3 +1145,25 @@ extern Buf slurm_persist_make_rc_msg(slurm_persist_conn_t *persist_conn,
 
 	return slurm_persist_msg_pack(persist_conn, &resp);
 }
+
+extern Buf slurm_persist_make_rc_msg_flags(slurm_persist_conn_t *persist_conn,
+					   uint32_t rc, char *comment,
+					   uint16_t flags,
+					   uint16_t ret_info)
+{
+	persist_rc_msg_t msg;
+	persist_msg_t resp;
+
+	memset(&msg, 0, sizeof(persist_rc_msg_t));
+	memset(&resp, 0, sizeof(persist_msg_t));
+
+	msg.rc = rc;
+	msg.flags = flags;
+	msg.comment = comment;
+	msg.ret_info = ret_info;
+
+	resp.msg_type = PERSIST_RC;
+	resp.data = &msg;
+
+	return slurm_persist_msg_pack(persist_conn, &resp);
+}
