@@ -1,14 +1,14 @@
 /*****************************************************************************\
- *  slurm_xlator.h - Definitions required to translate SLURM function names
+ *  slurm_xlator.h - Definitions required to translate Slurm function names
  *  to aliases containing  a prefix of "slurm_".
  *
- *  This is required because some SLURM functions have common names
+ *  This is required because some Slurm functions have common names
  *  (e.g. "debug" and "info"). If a user application defines these functions
- *  and uses SLURM APIs, they could link to the user function rather than
- *  the SLURM function. By renaming the functions, inappropriate linking
+ *  and uses Slurm APIs, they could link to the user function rather than
+ *  the Slurm function. By renaming the functions, inappropriate linking
  *  should be avoided.
  *
- *  All SLURM functions referenced from the switch, auth, and mpi plugins should
+ *  All Slurm functions referenced from the switch, auth, and mpi plugins should
  *  have aliases established. Functions not referenced from the plugins
  *  need not be aliased.
  *
@@ -33,11 +33,11 @@
  *             Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
  *
- *  This file is part of SLURM, a resource management program.
+ *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
  *  Please also read the included file: DISCLAIMER.
  *
- *  SLURM is free software; you can redistribute it and/or modify it under
+ *  Slurm is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
@@ -53,13 +53,13 @@
  *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
  *
- *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
+ *  Slurm is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with SLURM; if not, write to the Free Software Foundation, Inc.,
+ *  with Slurm; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
@@ -320,7 +320,6 @@
 #define	_xstrcat		slurm_xstrcat
 #define	_xstrncat		slurm_xstrncat
 #define	_xstrcatchar		slurm_xstrcatchar
-#define	_xslurm_strerrorcat	slurm_xslurm_strerrorcat
 #define	_xstrftimecat		slurm_xstrftimecat
 #define	_xiso8601timecat	slurm_xiso8601timecat
 #define	_xrfc5424timecat	slurm_xrfc5424timecat
@@ -330,7 +329,6 @@
 #define	xstrdup_printf		slurm_xstrdup_printf
 #define	xbasename		slurm_xbasename
 #define	_xstrsubstitute		slurm_xstrsubstitute
-#define	xstrstrip		slurm_xstrstrip
 #define	xshort_hostname		slurm_xshort_hostname
 #define xstring_is_whitespace   slurm_xstring_is_whitespace
 #define	xstrtolower		slurm_xstrtolower
@@ -360,9 +358,7 @@
 #define node_state_string_compact slurm_node_state_string_compact
 #define private_data_string	slurm_private_data_string
 #define accounting_enforce_string slurm_accounting_enforce_string
-#define conn_type_string	slurm_conn_type_string
 #define node_use_string		slurm_node_use_string
-#define bg_block_state_string	slurm_bg_block_state_string
 #define reservation_flags_string slurm_reservation_flags_string
 #define print_multi_line_string slurm_print_multi_line_string
 
@@ -392,9 +388,6 @@
 #define jobacctinfo_create      slurm_jobacctinfo_create
 #define jobacctinfo_destroy     slurm_jobacctinfo_destroy
 
-/* node_select.[ch] functions */
-#define destroy_select_ba_request	slurm_destroy_select_ba_request
-
 /* parse_config.[ch] functions */
 #define s_p_hashtbl_create	slurm_s_p_hashtbl_create
 #define s_p_hashtbl_destroy	slurm_s_p_hashtbl_destroy
